/* qsort - sort array a (dimension n) using quicksort
 * based on Bentley, CACM April 84
 * Comments use notation A[i], a (fictitious) array of things
 * that are of size elt_size.
 */
#include "local.h"
/* swapfn - swap  elt_size bytes  a <--> b (internal routine) 
 */
static void swapfn(a, b, elt_size)
	register char *a;	/* pointer to one element of A */
	register char *b;	/* pointer to another element of A */
	size_t elt_size;	/* size of each element */
	{
	register size_t i;
	char tmp;

	for (i = 0; i < elt_size; ++i, ++a, ++b)
		SWAP(*a, *b, tmp);
	}																				                                                  /*
.PE
.PS 25
/* iqsort - internal quicksort routine */
static void iqsort(p_lo, p_hi, elt_size, cmpfn)
	char *p_lo;			/* ptr to low element of (sub)array */
	char *p_hi;			/* ptr to high element of (sub)array */
	size_t elt_size;	/* size of each element */
	int (*cmpfn)();		/* comparison function ptr */
	{
	char *p_mid = p_lo + ((((p_hi - p_lo) / elt_size) / 2) * elt_size);
	register char *p_i;			/* pointer to A[i] */
	register char *p_lastlo;	/* pointer to A[lastlo] */

	swapfn(p_lo, p_mid, elt_size);	/* pick the middle element as pivot */
	p_lastlo = p_lo;
	for (p_i = p_lo + elt_size;  p_i <= p_hi; p_i += elt_size)
		{
		if ((*cmpfn)(p_lo, p_i) > 0)
			{
			p_lastlo += elt_size;
			swapfn(p_lastlo, p_i, elt_size);
			}
		}
	swapfn(p_lo, p_lastlo, elt_size);
	if (p_lo < p_lastlo && p_lo < p_lastlo - elt_size)
		iqsort(p_lo, p_lastlo - elt_size, elt_size, cmpfn);	/* lower */
	if (p_lastlo + elt_size < p_hi)
		iqsort(p_lastlo + elt_size, p_hi, elt_size, cmpfn);	/* upper */
	}																				                                                  /*
.PE
.PS 9
/* qsort - the callable entry point */
void qsort(a, n, size, pf)
	data_ptr a;		/* address of array A to be sorted */
	size_t n;		/* number of elements in A */
	size_t size;	/* size of each element */
	int (*pf)();	/* comparison function ptr */
	{
	if (n > 1)
		iqsort((char *)a, (char *)a + (n-1) * size, size, pf);
	}
